@@ -33,10 +33,13 @@
 from gettext import gettext as _
 import logging
 
-from gi.repository import Gtk, Gio, GLib, Gdk
+from gi.repository import Gtk, Gio, GLib, Gdk, GObject
 
 from gnomemusic import log
+from gnomemusic.inhibitsuspend import InhibitSuspend
 from gnomemusic.mpris import MediaPlayer2Service
+from gnomemusic.pauseonsuspend import PauseOnSuspend
+from gnomemusic.player import Player
 from gnomemusic.widgets.aboutdialog import AboutDialog
 from gnomemusic.window import Window
 
@@ -58,7 +61,12 @@ class Application(Gtk.Application):
 
         self._init_style()
         self._window = None
-        self._application_id = application_id
+
+        self._settings = Gio.Settings.new('org.gnome.Music')
+        self._player = Player(self)
+
+        InhibitSuspend(self)
+        PauseOnSuspend(self._player)
 
     def _init_style(self):
         css_provider = Gtk.CssProvider()
@@ -68,6 +76,26 @@ class Application(Gtk.Application):
         style_context.add_provider_for_screen(
             screen, css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
 
+    @GObject.Property(
+        type=Player, default=None, flags=GObject.ParamFlags.READABLE)
+    def player(self):
+        """Get application-wide music player.
+
+        :returns: the player
+        :rtype: Player
+        """
+        return self._player
+
+    @GObject.Property(
+        type=Gio.Settings, flags=GObject.ParamFlags.READABLE)
+    def settings(self):
+        """Get application-wide settings.
+
+        :returns: settings
+        :rtype: Gio.settings
+        """
+        return self._settings
+
     @log
     def _build_app_menu(self):
         action_entries = [
@@ -91,6 +119,7 @@ class Application(Gtk.Application):
     def _about(self, action, param):
         about = AboutDialog()
         about.props.transient_for = self._window
+        about.present()
 
     @log
     def do_startup(self):
@@ -105,9 +134,12 @@ class Application(Gtk.Application):
     def do_activate(self):
         if not self._window:
             self._window = Window(self)
-            self._window.set_default_icon_name(self._application_id)
-            if self._application_id == 'org.gnome.MusicDevel':
+            self._window.set_default_icon_name(self.props.application_id)
+            if self.props.application_id == 'org.gnome.MusicDevel':
                 self._window.get_style_context().add_class('devel')
             MediaPlayer2Service(self)
 
-        self._window.present()
+        # gtk_window_present does not work on Wayland.
+        # Use gtk_present_with_time as a workaround instead.
+        # See https://gitlab.gnome.org/GNOME/gtk/issues/624#note_10996
+        self._window.present_with_time(GLib.get_monotonic_time() / 1000)
