@@ -28,15 +28,12 @@ import logging
 from gettext import gettext as _, ngettext
 import gi
 gi.require_version('Gst', '1.0')
-gi.require_version('GstAudio', '1.0')
 gi.require_version('GstPbutils', '1.0')
-from gi.repository import Gtk, Gio, GObject, Gst, GstAudio, GstPbutils
+from gi.repository import Gtk, Gio, GObject, Gst, GstPbutils
 
 from gnomemusic import log
-from gnomemusic.playlists import Playlists
 
 logger = logging.getLogger(__name__)
-playlists = Playlists.get_default()
 
 
 class Playback(IntEnum):
@@ -61,15 +58,20 @@ class GstPlayer(GObject.GObject):
         return '<GstPlayer>'
 
     @log
-    def __init__(self):
+    def __init__(self, application):
+        """Initialize the GStreamer player
+
+        :param Application application: Application object
+        """
         super().__init__()
 
         Gst.init(None)
 
+        self._application = application
         self._duration = -1.
 
         self._missing_plugin_messages = []
-        self._settings = Gio.Settings.new('org.gnome.Music')
+        self._settings = application.props.settings
 
         self._player = Gst.ElementFactory.make('playbin3', 'player')
         self._bus = self._player.get_bus()
@@ -81,15 +83,12 @@ class GstPlayer(GObject.GObject):
         self._on_replaygain_setting_changed(
             None, self._settings.get_value('replaygain'))
 
-        self._bus.connect('message::state-changed', self._on_bus_state_changed)
+        self._bus.connect('message::async-done', self._on_async_done)
         self._bus.connect('message::error', self._on_bus_error)
         self._bus.connect('message::element', self._on_bus_element)
         self._bus.connect('message::eos', self._on_bus_eos)
-        self._bus.connect(
-            'message::duration-changed', self._on_duration_changed)
         self._bus.connect('message::new-clock', self._on_new_clock)
 
-        self._previous_state = Playback.STOPPED
         self.props.state = Playback.STOPPED
 
     @log
@@ -125,44 +124,27 @@ class GstPlayer(GObject.GObject):
             self._player.set_property("audio-filter", None)
 
     @log
-    def _on_new_clock(self, bus, message):
-        clock = message.parse_new_clock()
-        id_ = clock.new_periodic_id(0, 1 * Gst.SECOND)
-        clock.id_wait_async(id_, self._on_clock_tick, None)
-
-        # TODO: Workaround the first duration change not being emitted
-        # and hence smoothscale not being initialized properly.
-        if self.props.duration == -1.:
-            self._on_duration_changed(None, None)
-
-    @log
-    def _on_clock_tick(self, clock, time, id, data):
-        tick = time / Gst.SECOND
-        self.emit('clock-tick', tick)
-
-    @log
-    def _on_bus_state_changed(self, bus, message):
-        # Note: not all state changes are signaled through here, in
-        # particular transitions between Gst.State.READY and
-        # Gst.State.NULL are never async and thus don't cause a
-        # message. In practice, self means only Gst.State.PLAYING and
-        # Gst.State.PAUSED are.
-        current_state = self.props.state
-        if current_state == self._previous_state:
-            return
-
-        self._previous_state = current_state
-        self.notify('state')
-
-    @log
-    def _on_duration_changed(self, bus, message):
+    def _on_async_done(self, bus, message):
         success, duration = self._player.query_duration(
             Gst.Format.TIME)
 
         if success:
             self.props.duration = duration / Gst.SECOND
         else:
-            self.props.duration = -1.
+            self.props.duration = duration
+
+        self.notify('state')
+
+    @log
+    def _on_new_clock(self, bus, message):
+        clock = message.parse_new_clock()
+        id_ = clock.new_periodic_id(0, 1 * Gst.SECOND)
+        clock.id_wait_async(id_, self._on_clock_tick, None)
+
+    @log
+    def _on_clock_tick(self, clock, time, id, data):
+        tick = time / Gst.SECOND
+        self.emit('clock-tick', tick)
 
     @log
     def _on_bus_element(self, bus, message):
@@ -287,24 +269,6 @@ class GstPlayer(GObject.GObject):
         """
         self._duration = duration
 
-    @GObject.Property
-    def volume(self):
-        """Get current volume
-
-        :return: volume
-        :rtype: float
-        """
-        volume = self._player.get_volume(GstAudio.StreamVolumeFormat.LINEAR)
-        return volume
-
-    @volume.setter
-    def volume(self, volume):
-        """Set volume
-
-        :param float volume: The volume to set (0-10)
-        """
-        self._player.set_volume(GstAudio.StreamVolumeFormat.LINEAR, volume)
-
     @log
     def seek(self, seconds):
         """Seek to position
@@ -320,8 +284,8 @@ class GstPlayer(GObject.GObject):
     def _start_plugin_installation(
             self, missing_plugin_messages, confirm_search):
         install_ctx = GstPbutils.InstallPluginsContext.new()
-
-        install_ctx.set_desktop_id('org.gnome.Music.desktop')
+        application_id = self._application.props.application_id
+        install_ctx.set_desktop_id(application_id + '.desktop')
         install_ctx.set_confirm_search(confirm_search)
 
         startup_id = "_TIME{}".format(Gtk.get_current_event_time())
@@ -344,7 +308,8 @@ class GstPlayer(GObject.GObject):
     @log
     def _show_codec_confirmation_dialog(
             self, install_helper_name, missing_plugin_messages):
-        dialog = MissingCodecsDialog(self._parent_window, install_helper_name)
+        active_window = self._application.props.active_window
+        dialog = MissingCodecsDialog(active_window, install_helper_name)
 
         def on_dialog_response(dialog, response_type):
             if response_type == Gtk.ResponseType.ACCEPT:
