@@ -150,7 +150,6 @@ class MediaPlayer2Service(Server):
             <property name='Shuffle' type='b' access='readwrite'/>
             <property name='Metadata' type='a{sv}' access='read'>
             </property>
-            <property name='Volume' type='d' access='readwrite'/>
             <property name='Position' type='x' access='read'/>
             <property name='MinimumRate' type='d' access='read'/>
             <property name='MaximumRate' type='d' access='read'/>
@@ -219,13 +218,11 @@ class MediaPlayer2Service(Server):
         super().__init__(self.con, '/org/mpris/MediaPlayer2')
 
         self.app = app
-        self.player = app.get_active_window()._player
+        self.player = app.props.player
         self.player.connect(
             'song-changed', self._on_current_song_changed)
         self.player.connect('notify::state', self._on_player_state_changed)
         self.player.connect('notify::repeat-mode', self._on_repeat_mode_changed)
-        self.player.connect('volume-changed', self._on_volume_changed)
-        self.player.connect('prev-next-invalidated', self._on_prev_next_invalidated)
         self.player.connect('seek-finished', self._on_seek_finished)
         self.player.connect(
             'playlist-changed', self._on_player_playlist_changed)
@@ -240,6 +237,7 @@ class MediaPlayer2Service(Server):
         self._player_previous_type = None
         self._path_list = []
         self._metadata_list = []
+        self._previous_playback_status = "Stopped"
 
     @log
     def _get_playback_status(self):
@@ -468,39 +466,31 @@ class MediaPlayer2Service(Server):
 
     @log
     def _on_player_state_changed(self, klass, args):
+        playback_status = self._get_playback_status()
+        if playback_status == self._previous_playback_status:
+            return
+
+        self._previous_playback_status = playback_status
         self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE,
                                {
-                                   'PlaybackStatus': GLib.Variant('s', self._get_playback_status()),
+                                   'PlaybackStatus': GLib.Variant('s', playback_status),
                                },
                                [])
 
     @log
     def _on_repeat_mode_changed(self, player, param):
         self._update_songs_list()
+        has_next = self.player.props.has_next
+        has_previous = self.player.props.has_previous
         self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE,
                                {
+                                   'CanGoNext': GLib.Variant('b', has_next),
+                                   'CanGoPrevious': GLib.Variant('b', has_previous),
                                    'LoopStatus': GLib.Variant('s', self._get_loop_status()),
                                    'Shuffle': GLib.Variant('b', self.player.props.repeat_mode == RepeatMode.SHUFFLE),
                                },
                                [])
 
-    @log
-    def _on_volume_changed(self, player, data=None):
-        self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE,
-                               {
-                                   'Volume': GLib.Variant('d', self.player.get_volume()),
-                               },
-                               [])
-
-    @log
-    def _on_prev_next_invalidated(self, player, data=None):
-        self.PropertiesChanged(MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE,
-                               {
-                                   'CanGoNext': GLib.Variant('b', self.player.props.has_next),
-                                   'CanGoPrevious': GLib.Variant('b', self.player.props.has_previous),
-                               },
-                               [])
-
     @log
     def _on_seek_finished(self, player, position_second):
         self.Seeked(int(position_second * 1e6))
@@ -624,7 +614,8 @@ class MediaPlayer2Service(Server):
             self.player.props.current_song)
         current_song_index = self._path_list.index(current_song_path)
         goto_index = self._path_list.index(path)
-        self.player.play(goto_index - current_song_index)
+        song_offset = goto_index - current_song_index
+        self.player.play(song_offset=song_offset)
         return
 
     def TrackListReplaced(self, tracks, current_song):
@@ -682,6 +673,7 @@ class MediaPlayer2Service(Server):
 
     def GetAll(self, interface_name):
         if interface_name == MediaPlayer2Service.MEDIA_PLAYER2_IFACE:
+            application_id = self.app.props.application_id
             return {
                 'CanQuit': GLib.Variant('b', True),
                 'Fullscreen': GLib.Variant('b', False),
@@ -689,7 +681,7 @@ class MediaPlayer2Service(Server):
                 'CanRaise': GLib.Variant('b', True),
                 'HasTrackList': GLib.Variant('b', True),
                 'Identity': GLib.Variant('s', 'Music'),
-                'DesktopEntry': GLib.Variant('s', 'org.gnome.Music'),
+                'DesktopEntry': GLib.Variant('s', application_id),
                 'SupportedUriSchemes': GLib.Variant('as', [
                     'file'
                 ]),
@@ -708,7 +700,6 @@ class MediaPlayer2Service(Server):
                 'Rate': GLib.Variant('d', 1.0),
                 'Shuffle': GLib.Variant('b', self.player.props.repeat_mode == RepeatMode.SHUFFLE),
                 'Metadata': GLib.Variant('a{sv}', self._get_metadata()),
-                'Volume': GLib.Variant('d', self.player.get_volume()),
                 'Position': GLib.Variant('x', position_msecond),
                 'MinimumRate': GLib.Variant('d', 1.0),
                 'MaximumRate': GLib.Variant('d', 1.0),
@@ -745,10 +736,8 @@ class MediaPlayer2Service(Server):
             if property_name == 'Fullscreen':
                 pass
         elif interface_name == MediaPlayer2Service.MEDIA_PLAYER2_PLAYER_IFACE:
-            if property_name == 'Rate':
+            if property_name in ['Rate', 'Volume']:
                 pass
-            elif property_name == 'Volume':
-                self.player.set_volume(new_value)
             elif property_name == 'LoopStatus':
                 if new_value == 'None':
                     self.player.props.repeat_mode = RepeatMode.NONE
