@@ -30,16 +30,15 @@ import logging
 import time
 
 import gi
+gi.require_version('Grl', '0.3')
 gi.require_version('Gst', '1.0')
 gi.require_version('GstAudio', '1.0')
 gi.require_version('GstPbutils', '1.0')
-from gi.repository import Gio, GLib, GObject, Grl, Gst, GstPbutils
+from gi.repository import GObject, Grl, GstPbutils
 
 from gnomemusic import log
 from gnomemusic.gstplayer import GstPlayer, Playback
 from gnomemusic.grilo import grilo
-from gnomemusic.inhibitsuspend import InhibitSuspend
-from gnomemusic.pauseonsuspend import PauseOnSuspend
 from gnomemusic.playlists import Playlists
 from gnomemusic.scrobbler import LastFmScrobbler
 
@@ -91,12 +90,17 @@ class PlayerPlaylist(GObject.GObject):
 
     _nb_songs_max = 10
 
+    repeat_mode = GObject.Property(type=int, default=RepeatMode.NONE)
+
     def __repr__(self):
         return '<PlayerPlayList>'
 
     @log
     def __init__(self):
         super().__init__()
+
+        GstPbutils.pb_utils_init()
+
         self._songs = []
         self._shuffle_indexes = []
         self._current_index = 0
@@ -104,20 +108,22 @@ class PlayerPlaylist(GObject.GObject):
         self._type = -1
         self._id = -1
 
-        self._settings = Gio.Settings.new('org.gnome.Music')
-        self._settings.connect(
-            'changed::repeat', self._on_repeat_setting_changed)
-        self._repeat = self._settings.get_enum('repeat')
-
         self._validation_indexes = None
         self._discoverer = GstPbutils.Discoverer()
         self._discoverer.connect('discovered', self._on_discovered)
         self._discoverer.start()
 
+        self.connect("notify::repeat-mode", self._on_repeat_mode_changed)
+
     @log
-    def set_playlist(self, playlist_type, playlist_id, model, model_iter):
+    def set_playlist(self, playlist_type, playlist_id, model, model_iter=None):
         """Set a new playlist or change the song being played
 
+        If no song is requested (through model_iter), a song will be
+        automatically selected:
+        * the first song in a linear mode
+        * a random song in shuffle mode
+
         :param PlayerPlaylist.Type playlist_type: playlist type
         :param string playlist_id: unique identifer to recognize the playlist
         :param GtkListStore model: list of songs to play
@@ -126,6 +132,13 @@ class PlayerPlaylist(GObject.GObject):
         :return: True if the playlist has been updated. False otherwise
         :rtype: bool
         """
+        if not model_iter:
+            if self.props.repeat_mode == RepeatMode.SHUFFLE:
+                index = randrange(len(model))
+                model_iter = model.get_iter_from_string(str(index))
+            else:
+                model_iter = model.get_iter_first()
+
         path = model.get_path(model_iter)
         self._current_index = int(path.to_string())
 
@@ -148,7 +161,7 @@ class PlayerPlaylist(GObject.GObject):
         for row in model:
             self._songs.append([row[5], row[11]])
 
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             self._shuffle_indexes = list(range(len(self._songs)))
             shuffle(self._shuffle_indexes)
             self._shuffle_indexes.remove(self._current_index)
@@ -172,13 +185,13 @@ class PlayerPlaylist(GObject.GObject):
         :return: True if the index has changed. False otherwise.
         :rtype: bool
         """
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             shuffle = self._shuffle_indexes.index(self._current_index)
             self._current_index = self._shuffle_indexes[shuffle + song_offset]
             return True
 
         song_index = song_offset + self._current_index
-        if self._repeat == RepeatMode.ALL:
+        if self.props.repeat_mode == RepeatMode.ALL:
             song_index = song_index % len(self._songs)
 
         if(song_index >= len(self._songs)
@@ -214,7 +227,7 @@ class PlayerPlaylist(GObject.GObject):
                     return_index = self._current_index
                     break
 
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             index_l = self._shuffle_indexes.index(last_pos)
             self._shuffle_indexes.pop(index_l)
             self._shuffle_indexes = [
@@ -234,14 +247,14 @@ class PlayerPlaylist(GObject.GObject):
         """
         item = [song, ValidationStatus.PENDING]
         self._songs.insert(song_index, item)
-        if song_index >= self._current_index:
+        if song_index <= self._current_index:
             self._current_index += 1
 
         self._validate_song(song_index)
 
         # In the shuffle case, insert song at a random position which
         # has not been played yet.
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             index = self._shuffle_indexes.index(self._current_index)
             new_song_index = randrange(index, len(self._shuffle_indexes))
             self._shuffle_indexes.insert(new_song_index, song_index)
@@ -256,16 +269,12 @@ class PlayerPlaylist(GObject.GObject):
         if song_index < self._current_index:
             self._current_index -= 1
 
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             self._shuffle_indexes.remove(song_index)
             self._shuffle_indexes = [
                 index - 1 if index > song_index else index
                 for index in self._shuffle_indexes]
 
-    @log
-    def _on_repeat_setting_changed(self, settings, value):
-        self.props.repeat_mode = settings.get_enum('repeat')
-
     @log
     def _on_discovered(self, discoverer, info, error):
         url = info.get_uri()
@@ -308,12 +317,12 @@ class PlayerPlaylist(GObject.GObject):
         if not self.has_next():
             return -1
 
-        if self._repeat == RepeatMode.SONG:
+        if self.props.repeat_mode == RepeatMode.SONG:
             return self._current_index
-        if (self._repeat == RepeatMode.ALL
+        if (self.props.repeat_mode == RepeatMode.ALL
                 and self._current_index == (len(self._songs) - 1)):
             return 0
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             index = self._shuffle_indexes.index(self._current_index)
             return self._shuffle_indexes[index + 1]
         else:
@@ -324,12 +333,12 @@ class PlayerPlaylist(GObject.GObject):
         if not self.has_previous():
             return -1
 
-        if self._repeat == RepeatMode.SONG:
+        if self.props.repeat_mode == RepeatMode.SONG:
             return self._current_index
-        if (self._repeat == RepeatMode.ALL
+        if (self.props.repeat_mode == RepeatMode.ALL
                 and self._current_index == 0):
             return len(self._songs) - 1
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             shuffle_index = self._shuffle_indexes.index(self._current_index)
             return self._shuffle_indexes[shuffle_index - 1]
         else:
@@ -337,7 +346,7 @@ class PlayerPlaylist(GObject.GObject):
 
     @log
     def _validate_next_song(self):
-        if self._repeat == RepeatMode.SONG:
+        if self.props.repeat_mode == RepeatMode.SONG:
             return
 
         next_index = self._get_next_index()
@@ -346,7 +355,7 @@ class PlayerPlaylist(GObject.GObject):
 
     @log
     def _validate_previous_song(self):
-        if self._repeat == RepeatMode.SONG:
+        if self.props.repeat_mode == RepeatMode.SONG:
             return
 
         previous_index = self._get_previous_index()
@@ -360,11 +369,11 @@ class PlayerPlaylist(GObject.GObject):
         :return: True if there is a song. False otherwise.
         :rtype: bool
         """
-        if (self._repeat == RepeatMode.SHUFFLE
+        if (self.props.repeat_mode == RepeatMode.SHUFFLE
                 and self._shuffle_indexes):
             index = self._shuffle_indexes.index(self._current_index)
             return index < (len(self._shuffle_indexes) - 1)
-        if self._repeat != RepeatMode.NONE:
+        if self.props.repeat_mode != RepeatMode.NONE:
             return True
         return self._current_index < (len(self._songs) - 1)
 
@@ -375,11 +384,11 @@ class PlayerPlaylist(GObject.GObject):
         :return: True if there is a song. False otherwise.
         :rtype: bool
         """
-        if (self._repeat == RepeatMode.SHUFFLE
+        if (self.props.repeat_mode == RepeatMode.SHUFFLE
                 and self._shuffle_indexes):
             index = self._shuffle_indexes.index(self._current_index)
             return index > 0
-        if self._repeat != RepeatMode.NONE:
+        if self.props.repeat_mode != RepeatMode.NONE:
             return True
         return self._current_index > 0
 
@@ -447,34 +456,15 @@ class PlayerPlaylist(GObject.GObject):
         current_item = self._songs[self._current_index]
         return current_item[PlayerField.VALIDATION] != ValidationStatus.FAILED
 
-    @GObject.Property(type=int, default=RepeatMode.NONE)
-    def repeat_mode(self):
-        """Get repeat mode.
-
-        :returns: the repeat mode
-        :rtype: RepeatMode
-        """
-        return self._repeat
-
-    @repeat_mode.setter
-    def repeat_mode(self, mode):
-        """Set repeat mode.
-
-        :param RepeatMode mode: new repeat_mode
-        """
-        if mode == self._repeat:
-            return
-
-        if (mode == RepeatMode.SHUFFLE
+    @log
+    def _on_repeat_mode_changed(self, klass, param):
+        if (self.props.repeat_mode == RepeatMode.SHUFFLE
                 and self._songs):
             self._shuffle_indexes = list(range(len(self._songs)))
             shuffle(self._shuffle_indexes)
             self._shuffle_indexes.remove(self._current_index)
             self._shuffle_indexes.insert(0, self._current_index)
 
-        self._repeat = mode
-        self._settings.set_enum('repeat', mode)
-
     @GObject.Property(type=int, flags=GObject.ParamFlags.READABLE)
     def playlist_id(self):
         """Get playlist unique identifier.
@@ -510,7 +500,7 @@ class PlayerPlaylist(GObject.GObject):
         songs = []
         nb_songs = len(self._songs)
         current_index = self._current_index
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             current_index = self._shuffle_indexes.index(self._current_index)
 
         index_min = current_index - self._nb_songs_max
@@ -522,12 +512,12 @@ class PlayerPlaylist(GObject.GObject):
         first_index = max(index_min, 0)
         last_index = min(index_max, nb_songs)
 
-        if self._repeat == RepeatMode.SHUFFLE:
+        if self.props.repeat_mode == RepeatMode.SHUFFLE:
             indexes = self._shuffle_indexes[first_index:last_index]
         else:
             indexes = range(first_index, last_index)
 
-        if (self._repeat == RepeatMode.ALL
+        if (self.props.repeat_mode == RepeatMode.ALL
                 and (last_index - first_index) < (2 * self._nb_songs_max + 1)):
             offset_sup = min(
                 self._nb_songs_max - last_index + current_index + 1,
@@ -551,13 +541,10 @@ class Player(GObject.GObject):
     """
 
     __gsignals__ = {
-        'clock-tick': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         'playlist-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
-        'prev-next-invalidated': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'seek-finished': (GObject.SignalFlags.RUN_FIRST, None, (float,)),
         'song-changed': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         'song-validated': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
-        'volume-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     state = GObject.Property(type=int, default=Playback.STOPPED)
@@ -567,24 +554,28 @@ class Player(GObject.GObject):
         return '<Player>'
 
     @log
-    def __init__(self, parent_window):
-        super().__init__()
+    def __init__(self, application):
+        """Initialize the player
 
-        self._parent_window = parent_window
+        :param Application application: Application object
+        """
+        super().__init__()
 
         self._playlist = PlayerPlaylist()
         self._playlist.connect('song-validated', self._on_song_validated)
-        self._playlist.bind_property(
-            'repeat-mode', self, 'repeat-mode',
-            GObject.BindingFlags.SYNC_CREATE
-            | GObject.BindingFlags.BIDIRECTIONAL)
+
+        self._settings = application.props.settings
+        self._settings.connect(
+            'changed::repeat', self._on_repeat_setting_changed)
+
+        self._repeat = self._settings.get_enum('repeat')
+        self.bind_property(
+            'repeat-mode', self._playlist, 'repeat-mode',
+            GObject.BindingFlags.SYNC_CREATE)
 
         self._new_clock = True
 
-        Gst.init(None)
-        GstPbutils.pb_utils_init()
-
-        self._gst_player = GstPlayer()
+        self._gst_player = GstPlayer(application)
         self._gst_player.connect('clock-tick', self._on_clock_tick)
         self._gst_player.connect('eos', self._on_eos)
         self._gst_player.bind_property(
@@ -592,10 +583,6 @@ class Player(GObject.GObject):
         self._gst_player.bind_property(
             'state', self, 'state', GObject.BindingFlags.SYNC_CREATE)
 
-        root_window = parent_window.get_toplevel()
-        self._inhibit_suspend = InhibitSuspend(root_window, self)
-        self._pause_on_suspend = PauseOnSuspend(self)
-
         self._lastfm = LastFmScrobbler()
 
     @GObject.Property(
@@ -639,22 +626,19 @@ class Player(GObject.GObject):
 
     @log
     def _on_eos(self, klass):
-        def on_glib_idle():
-            self._playlist.next()
-            self.play()
-
         if self.props.has_next:
-            GLib.idle_add(on_glib_idle)
+            self.next()
         else:
             self.stop()
 
     @log
-    def play(self, song_offset=None):
+    def play(self, song_changed=True, song_offset=None):
         """Play a song.
 
-        If song_offset is None, load and play current song. Otherwise, load a
-        new song and play it.
+        Load a new song or resume playback depending on song_changed
+        value. If song_offset is defined, set a new song and play it.
 
+        :param bool song_changed: indicate if a new song must be loaded
         :param int song_offset: position relative to current song
         """
         if self.props.current_song is None:
@@ -664,9 +648,7 @@ class Player(GObject.GObject):
                 and not self._playlist.set_song(song_offset)):
             return False
 
-        url = self._playlist.props.current_song.get_url()
-        if (url != self._gst_player.props.url
-                or self._repeat == RepeatMode.SONG):
+        if song_changed is True:
             self._load(self._playlist.props.current_song)
 
         self._gst_player.props.state = Playback.PLAYING
@@ -710,10 +692,10 @@ class Player(GObject.GObject):
         if self.props.state == Playback.PLAYING:
             self.pause()
         else:
-            self.play()
+            self.play(False)
 
     @log
-    def set_playlist(self, playlist_type, playlist_id, model, iter_):
+    def set_playlist(self, playlist_type, playlist_id, model, iter_=None):
         """Set a new playlist or change the song being played.
 
         :param PlayerPlaylist.Type playlist_type: playlist type
@@ -724,9 +706,6 @@ class Player(GObject.GObject):
         playlist_changed = self._playlist.set_playlist(
             playlist_type, playlist_id, model, iter_)
 
-        if self.props.state == Playback.PLAYING:
-            self.emit('prev-next-invalidated')
-
         if playlist_changed:
             self.emit('playlist-changed')
 
@@ -741,7 +720,7 @@ class Player(GObject.GObject):
         """
         current_index = self._playlist.change_position(prev_pos, new_pos)
         if current_index >= 0:
-            self.emit('prev-next-invalidated')
+            self.emit('playlist-changed')
         return current_index
 
     @log
@@ -759,7 +738,6 @@ class Player(GObject.GObject):
                 self.stop()
         self._playlist.remove_song(song_index)
         self.emit('playlist-changed')
-        self.emit('prev-next-invalidated')
 
     @log
     def add_song(self, song, song_index):
@@ -769,7 +747,6 @@ class Player(GObject.GObject):
         """
         self._playlist.add_song(song, song_index)
         self.emit('playlist-changed')
-        self.emit('prev-next-invalidated')
 
     @log
     def _on_song_validated(self, playlist, index, status):
@@ -815,23 +792,28 @@ class Player(GObject.GObject):
             if (percentage > 0.5
                     and self._new_clock):
                 self._new_clock = False
-                # FIXME: we should not need to update static
+                # FIXME: we should not need to update smart
                 # playlists here but removing it may introduce
                 # a bug. So, we keep it for the time being.
-                playlists.update_all_static_playlists()
+                playlists.update_all_smart_playlists()
                 grilo.bump_play_count(current_song)
                 grilo.set_last_played(current_song)
 
-        self.emit('clock-tick', int(position))
+    @log
+    def _on_repeat_setting_changed(self, settings, value):
+        self.props.repeat_mode = settings.get_enum('repeat')
 
-    @GObject.Property(type=int)
+    @GObject.Property(type=int, default=RepeatMode.NONE)
     def repeat_mode(self):
         return self._repeat
 
     @repeat_mode.setter
     def repeat_mode(self, mode):
+        if mode == self._repeat:
+            return
+
         self._repeat = mode
-        self.emit('prev-next-invalidated')
+        self._settings.set_enum('repeat', mode)
 
     @GObject.Property(
         type=Grl.Media, default=None, flags=GObject.ParamFlags.READABLE)
@@ -888,15 +870,6 @@ class Player(GObject.GObject):
             self._gst_player.seek(position_second)
             self.emit('seek-finished', position_second)
 
-    @log
-    def get_volume(self):
-        return self._gst_player.props.volume
-
-    @log
-    def set_volume(self, rate):
-        self._gst_player.props.volume = rate
-        self.emit('volume-changed')
-
     @log
     def get_mpris_playlist(self):
         """Get recent and next songs from the current playlist.
