@@ -25,6 +25,9 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
+import gi
+gi.require_version('Tracker', '2.0')
+
 from gi.repository import GLib, Tracker
 from gnomemusic import log
 import os
@@ -35,7 +38,6 @@ import time
 sparql_midnight_dateTime_format = "%Y-%m-%dT00:00:00Z"
 
 SECONDS_PER_DAY = 86400
-PUNCTUATION_FILTER = " !\\\"#$%&'()*+,-./:;<=>?@[\\\\]^_`{|}~"
 
 
 class Query():
