@@ -25,16 +25,17 @@
 from enum import IntEnum
 
 from gettext import gettext as _
+from gi.repository import Gio
 
 
 class View(IntEnum):
     """Enum for views"""
-    ALBUM = 0
-    ARTIST = 1
-    SONG = 2
-    PLAYLIST = 3
-    SEARCH = 4
-    EMPTY_SEARCH = 5
+    EMPTY = 0
+    ALBUM = 1
+    ARTIST = 2
+    SONG = 3
+    PLAYLIST = 4
+    SEARCH = 5
 
 
 def get_album_title(item):
@@ -48,10 +49,11 @@ def get_album_title(item):
     :return: The album title
     :rtype: string
     """
-    album = item.get_album()
-
-    if not album:
+    if item.is_container():
         album = get_media_title(item)
+    else:
+        album = (item.get_album()
+                 or _("Unknown album"))
 
     return album
 
@@ -83,8 +85,32 @@ def get_media_title(item):
     :rtype:
     """
 
-    return (item.get_title()
-            or _("Untitled"))
+    title = item.get_title()
+
+    if not title:
+        url = item.get_url()
+        file_ = Gio.File.new_for_uri(url)
+        fileinfo = file_.query_info(
+            "standard::display-name", Gio.FileQueryInfoFlags.NONE, None)
+        title = fileinfo.get_display_name()
+        title = title.replace("_", " ")
+
+    return title
+
+
+def get_media_year(item):
+    """Returns the year when the media was created.
+
+    :param item: A Grilo Media object
+    :return: The creation year or None if not defined
+    :rtype: string
+    """
+    date = item.get_creation_date()
+
+    if not date:
+        return None
+
+    return str(date.get_year())
 
 
 def seconds_to_string(duration):
