@@ -167,7 +167,7 @@ class ArtistsView(BaseView):
             self._sidebar.emit('row-activated', row)
 
     @log
-    def populate(self):
+    def _populate(self, data=None):
         """Populates the view"""
         self._window.notifications_popup.push_loading()
         grilo.populate_artists(self._offset, self._add_item)
@@ -179,7 +179,7 @@ class ArtistsView(BaseView):
         modifiers = Gtk.accelerator_get_default_mod_mask()
         if ((state & modifiers) == Gdk.ModifierType.CONTROL_MASK
                 and not self.props.selection_mode):
-            self._on_selection_mode_request()
+            self.props.selection_mode = True
 
     @log
     def _on_selection_changed(self, widget, value, data=None):
