@@ -128,17 +128,13 @@ class BaseView(Gtk.Stack):
 
     @log
     def _populate(self, data=None):
-        self.populate()
+        pass
 
     @log
     def _on_selection_mode_changed(self, widget, data=None):
         if not self.props.selection_mode:
             self.unselect_all()
 
-    @log
-    def populate(self):
-        pass
-
     @log
     def _retrieval_finished(self, klass):
         self.model[klass.iter][4] = klass.pixbuf
@@ -147,10 +143,6 @@ class BaseView(Gtk.Stack):
     def _on_item_activated(self, widget, id, path):
         pass
 
-    @log
-    def _on_selection_mode_request(self, *args):
-        self.props.selection_mode = not self.props.selection_mode
-
     @log
     def get_selected_songs(self, callback):
         callback([])
