@@ -24,12 +24,12 @@
 
 from gettext import gettext as _
 
-from gi.repository import Gio, GLib, GObject, Gtk, Pango
+from gi.repository import Gdk, Gio, GLib, GObject, Gtk, Pango
 
 from gnomemusic import log
 from gnomemusic.grilo import grilo
 from gnomemusic.player import ValidationStatus, PlayerPlaylist
-from gnomemusic.playlists import Playlists, StaticPlaylists
+from gnomemusic.playlists import Playlists
 from gnomemusic.views.baseview import BaseView
 from gnomemusic.widgets.notificationspopup import PlaylistNotification
 from gnomemusic.widgets.playlistcontextmenu import PlaylistContextMenu
@@ -44,11 +44,6 @@ playlists = Playlists.get_default()
 class PlaylistView(BaseView):
     """Main view for playlists"""
 
-    __gsignals__ = {
-        'playlists-loaded': (GObject.SignalFlags.RUN_FIRST, None, ()),
-        'playlist-songs-loaded': (GObject.SignalFlags.RUN_FIRST, None, ()),
-    }
-
     def __repr__(self):
         return '<PlaylistView>'
 
@@ -122,6 +117,7 @@ class PlaylistView(BaseView):
         self._iter_to_clean_model = None
         self._current_playlist = None
         self._current_playlist_index = None
+        self._plays_songs_on_activation = False
         self.pls_todelete = {}
         self._songs_todelete = {}
         self._songs_count = 0
@@ -154,12 +150,17 @@ class PlaylistView(BaseView):
         self._view.set_model(self.model)
         self._view.set_activate_on_single_click(True)
         self._view.get_selection().set_mode(Gtk.SelectionMode.SINGLE)
+
         self._view.connect('row-activated', self._on_song_activated)
-        self._view.connect('button-press-event', self._on_view_clicked)
         self._view.connect('drag-begin', self._drag_begin)
         self._view.connect('drag-end', self._drag_end)
         self._song_drag = {'active': False}
 
+        self._controller = Gtk.GestureMultiPress().new(self._view)
+        self._controller.props.propagation_phase = Gtk.PropagationPhase.CAPTURE
+        self._controller.props.button = Gdk.BUTTON_SECONDARY
+        self._controller.connect("pressed", self._on_view_right_clicked)
+
         view_container.add(self._view)
 
     @log
@@ -261,6 +262,8 @@ class PlaylistView(BaseView):
 
         iter_ = self.model.get_iter_from_string(str(position))
         self.model[iter_][10] = True
+        path = self.model.get_path(iter_)
+        self._view.scroll_to_cell(path, None, False, 0., 0.)
         if self.model[iter_][8] != self._error_icon_name:
             self._iter_to_clean = iter_.copy()
             self._iter_to_clean_model = self.model
@@ -268,8 +271,9 @@ class PlaylistView(BaseView):
         return False
 
     @log
-    def _add_playlist_item(self, source, param, playlist, remaining=0,
-                           data=None):
+    def _add_playlist_item(
+            self, source, param, playlist, remaining=0,
+            select_playlist_id=None):
         """Grilo.populate_playlists callback.
 
         Add all playlists found by Grilo to sidebar
@@ -278,17 +282,22 @@ class PlaylistView(BaseView):
         :param int param: param
         :param GrlMedia playlist: playlist to add
         :param int remaining: next playlist_id or zero if None
-        :param data: associated data
+        :param str select_playlist_id: playlist id to select on load
         """
         if not playlist:
             self._window.notifications_popup.pop_loading()
-            self.emit('playlists-loaded')
+            if not self._sidebar.get_selected_row():
+                first_row = self._sidebar.get_row_at_index(0)
+                self._sidebar.select_row(first_row)
+                first_row.emit('activate')
             return
 
-        self._add_playlist_to_sidebar(playlist, None)
+        select_playlist = (playlist.get_id() == select_playlist_id)
+        self._add_playlist_to_sidebar(playlist, None, select_playlist)
 
     @log
-    def _add_playlist_to_sidebar(self, playlist, index=None):
+    def _add_playlist_to_sidebar(
+            self, playlist, index=None, select_playlist=False):
         """Add a playlist to sidebar
 
         :param GrlMedia playlist: playlist to add
@@ -296,7 +305,7 @@ class PlaylistView(BaseView):
         """
         if index is None:
             index = -1
-        if playlists.is_static_playlist(playlist):
+        if playlists.is_smart_playlist(playlist):
             index = 0
 
         title = utils.get_media_title(playlist)
@@ -308,9 +317,9 @@ class PlaylistView(BaseView):
         self._sidebar.insert(row, index)
         self._offset += 1
 
-        if len(self._sidebar) == 1:
+        if select_playlist:
             self._sidebar.select_row(row)
-            self._sidebar.emit('row-activated', row)
+            row.emit('activate')
 
     @log
     def _on_song_validated(self, player, index, status):
@@ -338,43 +347,38 @@ class PlaylistView(BaseView):
         """
         def activate_song():
             if self._song_drag['active']:
-                return
+                return GLib.SOURCE_REMOVE
 
             if self._star_handler.star_renderer_click:
                 self._star_handler.star_renderer_click = False
-                return
+                return GLib.SOURCE_REMOVE
 
-            _iter = self.model.get_iter(path)
-            if self.model[_iter][8] != self._error_icon_name:
-                self.player.set_playlist(
-                    PlayerPlaylist.Type.PLAYLIST,
-                    self._current_playlist.get_id(), self.model, _iter)
-                self.player.play()
+            _iter = None
+            if path:
+                _iter = self.model.get_iter(path)
+            playlist_id = self._current_playlist.get_id()
+            self.player.set_playlist(
+                PlayerPlaylist.Type.PLAYLIST, playlist_id, self.model, _iter)
+            self.player.play()
+
+            return GLib.SOURCE_REMOVE
 
         # 'row-activated' signal is emitted before 'drag-begin' signal.
         # Need to wait to check if drag and drop operation is active.
         GLib.idle_add(activate_song)
 
     @log
-    def _on_view_clicked(self, treeview, event):
-        """Right click on self._view displays a context menu
-
-        :param Gtk.TreeView treeview: self._view
-        :param Gdk.EventButton event: clicked event
-        """
-        if not event.triggers_context_menu():
-            return
-
-        (coord_x, coord_y) = event.get_coords()
-        path, col, cell_x, cell_y = treeview.get_path_at_pos(coord_x, coord_y)
+    def _on_view_right_clicked(self, gesture, n_press, x, y):
+        (path, column, cell_x, cell_y) = self._view.get_path_at_pos(x, y)
         self._view.get_selection().select_path(path)
+        row_height = self._view.get_cell_area(path, None).height
 
-        rect = self._view.get_visible_rect()
-        rect.x = coord_x - rect.width / 2.0
-        rect.y = coord_y - rect.height + 5
+        rect = Gdk.Rectangle()
+        rect.x = x
+        rect.y = y - cell_y + 0.5 * row_height
 
-        self._song_popover.set_relative_to(self._view)
-        self._song_popover.set_pointing_to(rect)
+        self._song_popover.props.relative_to = self._view
+        self._song_popover.props.pointing_to = rect
         self._song_popover.popup()
 
     @log
@@ -438,8 +442,7 @@ class PlaylistView(BaseView):
     def _play_song(self, menuitem, data=None):
         model, _iter = self._view.get_selection().get_selected()
         path = model.get_path(_iter)
-        cols = self._view.get_columns()
-        self._view.emit('row-activated', path, cols[0])
+        self._view.emit('row-activated', path, None)
 
     @log
     def _add_song_to_playlist(self, menuitem, data=None):
@@ -478,59 +481,47 @@ class PlaylistView(BaseView):
             if (str(playlist_id) == playlist.get_id()
                     and self._is_current_playlist(playlist)):
                 self._star_handler.star_renderer_click = False
-                GLib.idle_add(self._on_playlist_activated, self._sidebar, row)
+                self._on_playlist_activated(self._sidebar, row)
                 break
 
     @log
     def activate_playlist(self, playlist_id):
+        """Selects and starts playing a playlist.
 
-        def find_and_activate_playlist():
-            for row in self._sidebar:
-                if row.playlist.get_id() == playlist_id:
-                    selection = self._sidebar.get_selected_row()
-                    if selection.get_index() == row.get_index():
-                        self._on_play_activate(None)
-                    else:
-                        self._sidebar.select_row(row)
-                        handler = 0
+        If the view has not been populated yet, populate it and then
+        select the requested playlist. Otherwise, directly select the
+        requested playlist and start playing.
 
-                        def songs_loaded_callback(view):
-                            self.disconnect(handler)
-                            self._on_play_activate(None)
+        :param str playlist_id: requested playlist id
+        """
+        if not self._init:
+            self._plays_songs_on_activation = True
+            self._populate(playlist_id)
+            return
 
-                        handler = self.connect('playlist-songs-loaded',
-                                               songs_loaded_callback)
-                        self._sidebar.emit('row-activated', row)
+        playlist_row = None
+        for row in self._sidebar:
+            if row.playlist.get_id() == playlist_id:
+                playlist_row = row
+                break
 
-                    return
+        if not playlist_row:
+            return
 
-        if self._init:
-            find_and_activate_playlist()
+        selection = self._sidebar.get_selected_row()
+        if selection.get_index() == row.get_index():
+            self._on_play_activate(None)
         else:
-            handler = 0
-
-            def playlists_loaded_callback(view):
-                self.disconnect(handler)
-                def_handler = 0
-
-                def songs_loaded_callback(view):
-                    self.disconnect(def_handler)
-                    find_and_activate_playlist()
-
-                # Skip load of default playlist
-                def_handler = self.connect('playlist-songs-loaded',
-                                           songs_loaded_callback)
-
-            handler = self.connect('playlists-loaded',
-                                   playlists_loaded_callback)
-
-            self._populate()
+            self._plays_songs_on_activation = True
+            self._sidebar.select_row(row)
+            row.emit('activate')
 
     @log
     def remove_playlist(self):
         """Removes the current selected playlist"""
-        if not self._current_playlist_is_protected():
-            self._stage_playlist_for_deletion(None)
+        if playlists.is_smart_playlist(self._current_playlist):
+            return
+        self._stage_playlist_for_deletion(None)
 
     @log
     def _on_playlist_activated(self, sidebar, row, data=None):
@@ -555,16 +546,11 @@ class PlaylistView(BaseView):
         self._pl_ctrls.props.display_songs_count = False
         grilo.populate_playlist_songs(playlist, self._add_song)
 
-        if self._current_playlist_is_protected():
-            self._playlist_delete_action.set_enabled(False)
-            self._playlist_rename_action.set_enabled(False)
-            self._remove_song_action.set_enabled(False)
-            self._view.set_reorderable(False)
-        else:
-            self._playlist_delete_action.set_enabled(True)
-            self._playlist_rename_action.set_enabled(True)
-            self._remove_song_action.set_enabled(True)
-            self._view.set_reorderable(True)
+        protected_pl = playlists.is_smart_playlist(self._current_playlist)
+        self._playlist_delete_action.set_enabled(not protected_pl)
+        self._playlist_rename_action.set_enabled(not protected_pl)
+        self._remove_song_action.set_enabled(not protected_pl)
+        self._view.set_reorderable(not protected_pl)
 
     @log
     def _add_song(self, source, param, song, remaining=0, data=None):
@@ -582,6 +568,13 @@ class PlaylistView(BaseView):
         if remaining == 0:
             self._view.set_model(self.model)
             self._pl_ctrls.props.display_songs_count = True
+            if self._plays_songs_on_activation:
+                first_iter = self.model.get_iter_first()
+                self.player.set_playlist(
+                    PlayerPlaylist.Type.PLAYLIST,
+                    self._current_playlist.get_id(), self.model, first_iter)
+                self.player.play()
+                self._plays_songs_on_activation = False
 
     @log
     def _add_song_to_model(self, song, model, index=-1):
@@ -590,7 +583,6 @@ class PlaylistView(BaseView):
         :param Gtk.ListStore model: model
         """
         if not song:
-            self.emit('playlist-songs-loaded')
             return None
 
         title = utils.get_media_title(song)
@@ -605,19 +597,7 @@ class PlaylistView(BaseView):
 
     @log
     def _on_play_activate(self, menuitem, data=None):
-        _iter = self.model.get_iter_first()
-        if not _iter:
-            return
-
-        selection = self._view.get_selection()
-        selection.select_path(self.model.get_path(_iter))
-        cols = self._view.get_columns()
-        self._view.emit('row-activated', self.model.get_path(_iter), cols[0])
-
-    @log
-    def _current_playlist_is_protected(self):
-        current_playlist_id = self._current_playlist.get_id()
-        return current_playlist_id in StaticPlaylists().get_ids()
+        self._view.emit('row-activated', None, None)
 
     @log
     def _is_current_playlist(self, playlist):
@@ -682,7 +662,7 @@ class PlaylistView(BaseView):
 
         if row_next:
             self._sidebar.select_row(row_next)
-            self._sidebar.emit('row-activated', row_next)
+            row_next.emit('activate')
 
         self._create_notification(
             PlaylistNotification.Type.PLAYLIST, playlist_id)
@@ -701,18 +681,20 @@ class PlaylistView(BaseView):
 
         else:
             song_todelete = self._songs_todelete[media_id]
-            playlist = song_todelete['playlist']
-            if (self._current_playlist
-                    and playlist.get_id() == self._current_playlist.get_id()):
-                iter_ = self._add_song_to_model(
-                    song_todelete['song'], self.model, song_todelete['index'])
-                playlist_id = self._current_playlist.get_id()
-                if self.player.playing_playlist(
-                        PlayerPlaylist.Type.PLAYLIST, playlist_id):
-                    song = self.model[iter_][5]
-                    path = self.model.get_path(iter_)
-                    self.player.add_song(song, int(path.to_string()))
             self._songs_todelete.pop(media_id)
+            if not self._is_current_playlist(song_todelete['playlist']):
+                return
+
+            iter_ = self._add_song_to_model(
+                song_todelete['song'], self.model, song_todelete['index'])
+
+            playlist_id = self._current_playlist.get_id()
+            if not self.player.playing_playlist(
+                    PlayerPlaylist.Type.PLAYLIST, playlist_id):
+                return
+
+            path = self.model.get_path(iter_)
+            self.player.add_song(self.model[iter_][5], int(path.to_string()))
 
     @log
     def _finish_pending_deletion(self, playlist_notification):
@@ -744,7 +726,7 @@ class PlaylistView(BaseView):
     @log
     def _on_playlist_renamed(self, arguments, new_name):
         selection = self._sidebar.get_selected_row()
-        selection.get_child().props.label = new_name
+        selection.props.text = new_name
 
         pl_torename = selection.playlist
         pl_torename.set_title(new_name)
@@ -752,7 +734,13 @@ class PlaylistView(BaseView):
 
     @log
     def _on_playlist_created(self, playlists, playlist):
-        """Add new playlist to sidebar"""
+        """Adds new playlist to sidebar
+
+        If the sidebar has not been populated yet, it has no effect:
+        the playlist will be displayed once the playlists are loaded.
+        """
+        if not self._init:
+            return
         self._add_playlist_to_sidebar(playlist)
 
     @log
@@ -780,10 +768,11 @@ class PlaylistView(BaseView):
         self._update_songs_count(self._songs_count - 1)
 
     @log
-    def populate(self):
+    def _populate(self, data=None):
         """Populate sidebar.
         Do not reload playlists already displayed.
         """
         self._window.notifications_popup.push_loading()
-        grilo.populate_playlists(self._offset, self._add_playlist_item)
+        grilo.populate_playlists(
+            self._offset, self._add_playlist_item, -1, data)
         self._init = True
