@@ -55,9 +55,6 @@ class SearchView(BaseView):
     def __init__(self, window, player):
         super().__init__('search', None, window)
 
-        # FIXME: Searchbar handling does not belong here.
-        self._searchbar = window._searchbar
-
         self._add_list_renderers()
         self.player = player
         self._head_iters = [None, None, None, None]
@@ -81,6 +78,9 @@ class SearchView(BaseView):
 
         self._items_found = None
 
+        self._search_mode_active = False
+        self.connect("notify::search-state", self._on_search_state_changed)
+
     @log
     def _setup_view(self):
         view_container = Gtk.ScrolledWindow(hexpand=True, vexpand=True)
@@ -103,8 +103,6 @@ class SearchView(BaseView):
 
     @log
     def _back_button_clicked(self, widget, data=None):
-        self._searchbar.reveal(True, False)
-
         if self.get_visible_child() == self._artist_albums_widget:
             self._artist_albums_widget.destroy()
             self._artist_albums_widget = None
@@ -113,6 +111,7 @@ class SearchView(BaseView):
                 self._window.views[View.ALBUM]._grid)
 
         self.set_visible_child(self._grid)
+        self.props.search_mode_active = True
         self._headerbar.props.state = HeaderBar.State.MAIN
 
     @log
@@ -141,7 +140,7 @@ class SearchView(BaseView):
             self._headerbar.props.title = title
             self._headerbar.props.subtitle = artist
             self.set_visible_child(self._album_widget)
-            self.props.search_state = Search.State.NONE
+            self.props.search_mode_active = False
 
         elif self.model[_iter][12] == 'artist':
             artist = self.model[_iter][2]
@@ -162,7 +161,7 @@ class SearchView(BaseView):
             self._headerbar.props.title = artist
             self._headerbar.props.subtitle = None
             self.set_visible_child(self._artist_albums_widget)
-            self.props.search_state = Search.State.NONE
+            self.props.search_mode_active = False
         elif self.model[_iter][12] == 'song':
             if self.model[_iter][11] != ValidationStatus.FAILED:
                 c_iter = self._songs_model.convert_child_iter_to_iter(_iter)[1]
@@ -183,7 +182,7 @@ class SearchView(BaseView):
         modifiers = Gtk.accelerator_get_default_mod_mask()
         if (state & modifiers == Gdk.ModifierType.CONTROL_MASK
                 and not self.props.selection_mode):
-            self._on_selection_mode_request()
+            self.props.selection_mode = True
 
         if (self.selection_mode
                 and not self._star_handler.star_renderer_click):
@@ -214,6 +213,37 @@ class SearchView(BaseView):
         cells[4].props.visible = self.props.selection_mode
         col.queue_resize()
 
+    @log
+    def _on_search_state_changed(self, klass, param):
+        # If a search is triggered when selection mode is activated,
+        # reset the number of selected items.
+        if (self.props.selection_mode
+                and self.props.search_state != Search.State.NONE):
+            self.props.selected_items_count = 0
+
+    @GObject.Property(type=bool, default=False)
+    def search_mode_active(self):
+        """Get search mode status.
+
+        :returns: the search mode status
+        :rtype: bool
+        """
+        return self._search_mode_active
+
+    @search_mode_active.setter
+    def search_mode_active(self, value):
+        """Set search mode status.
+
+        :param bool mode: new search mode
+        """
+        # FIXME: search_mode_active should not change search_state.
+        # This is necessary because Search state cannot interact with
+        # the child views.
+        self._search_mode_active = value
+        if (not self._search_mode_active
+                and self.get_visible_child() == self._grid):
+            self.props.search_state = Search.State.NONE
+
     @log
     def _add_search_item(self, source, param, item, remaining=0, data=None):
         if not item:
@@ -418,7 +448,7 @@ class SearchView(BaseView):
             cell.props.visible = False
 
     @log
-    def populate(self):
+    def _populate(self, data=None):
         self._init = True
         self._headerbar.props.state = HeaderBar.State.MAIN
 
