@@ -76,9 +76,12 @@ class SongsView(BaseView):
         self._view.props.model = self.model
         self._view.props.activate_on_single_click = True
 
+        self._ctrl = Gtk.GestureMultiPress().new(self._view)
+        self._ctrl.props.propagation_phase = Gtk.PropagationPhase.CAPTURE
+        self._ctrl.connect("released", self._on_view_clicked)
+
         self._view.get_selection().props.mode = Gtk.SelectionMode.SINGLE
         self._view.connect('row-activated', self._on_item_activated)
-        self._view.connect('button-release-event', self._on_view_clicked)
 
         view_container.add(self._view)
 
@@ -170,7 +173,7 @@ class SongsView(BaseView):
                 and not self.props.selection_mode):
             self.model.clear()
             self._offset = 0
-            self.populate()
+            self._populate()
             grilo.changes_pending['Songs'] = False
 
     @log
@@ -208,21 +211,18 @@ class SongsView(BaseView):
         self.player.play()
 
     @log
-    def _on_view_clicked(self, treeview, event):
-        """Ctrl+click on self._view triggers selection mode.
-
-        :param Gtk.TreeView treeview: self._view
-        :param Gdk.EventButton event: clicked event
-        """
+    def _on_view_clicked(self, gesture, n_press, x, y):
+        """Ctrl+click on self._view triggers selection mode."""
+        _, state = Gtk.get_current_event_state()
         modifiers = Gtk.accelerator_get_default_mod_mask()
-        if ((event.get_state() & modifiers) == Gdk.ModifierType.CONTROL_MASK
+        if (state & modifiers == Gdk.ModifierType.CONTROL_MASK
                 and not self.props.selection_mode):
-            self._on_selection_mode_request()
+            self.props.selection_mode = True
 
-        if (self.selection_mode
-                and not self._star_handler.star_renderer_click):
-            path, col, cell_x, cell_y = treeview.get_path_at_pos(
-                *event.get_coords())
+        # FIXME: In selection mode, star clicks might still trigger
+        # activation.
+        if self.props.selection_mode:
+            path, col, cell_x, cell_y = self._view.get_path_at_pos(x, y)
             iter_ = self.model.get_iter(path)
             self.model[iter_][6] = not self.model[iter_][6]
 
@@ -276,7 +276,7 @@ class SongsView(BaseView):
             [utils.get_media_title(item), artist, item, item.get_favourite()])
 
     @log
-    def populate(self):
+    def _populate(self, data=None):
         """Populates the view"""
         self._init = True
         if grilo.tracker:
