@@ -30,7 +30,7 @@ from gnomemusic.playlists import Playlists
 import gnomemusic.utils as utils
 
 
-@Gtk.Template(resource_path="/org/gnome/Music/PlaylistDialog.ui")
+@Gtk.Template(resource_path="/org/gnome/Music/ui/PlaylistDialog.ui")
 class PlaylistDialog(Gtk.Dialog):
     """Dialog for adding items to a playlist"""
 
@@ -115,7 +115,7 @@ class PlaylistDialog(Gtk.Dialog):
 
     @log
     def _add_item_to_model(self, item):
-        """Adds (non-static only) playlists to the model"""
+        """Adds (non-smart only) playlists to the model"""
 
         # Hide playlists that are going to be deleted
         if item.get_id() in self._playlists_todelete_ids:
