@@ -44,6 +44,8 @@ class BaseModelColumns(IntEnum):
 
 class BaseManager(GObject.GObject):
 
+    default_value = GObject.Property(type=int, default=1)
+
     def __repr__(self):
         return '<BaseManager>'
 
@@ -69,10 +71,13 @@ class BaseManager(GObject.GObject):
                 ['search_composer', _("Composer"), ''],
                 ['search_track', _("Track Title"), ''],
             ]
+
         for value in self.values:
             iter_ = model.append()
             model[iter_][0, 1, 2] = value
-        self.selected_id = self.values[1][BaseModelColumns.ID]
+
+        value = self.values[self.props.default_value]
+        self.selected_id = value[BaseModelColumns.ID]
 
     @GObject.Property
     def active(self):
@@ -88,17 +93,15 @@ class BaseManager(GObject.GObject):
 
         if selected_value != []:
             selected_value = selected_value[0]
+            selected_index = self.values.index(selected_value)
             self.selected_id = selected_value[BaseModelColumns.ID]
 
-            # If selected values has first entry then it is a default
-            # value. No need to set the tag there.
-            value_id = selected_value[BaseModelColumns.ID]
-            if (value_id != 'search_all'
-                    and value_id != 'grl-tracker-source'):
+            # If selected value is the default one, hide the tag.
+            if selected_index == self.props.default_value:
+                self.entry.remove_tag(self._tag)
+            else:
                 self._tag.set_label(selected_value[BaseModelColumns.NAME])
                 self.entry.add_tag(self._tag)
-            else:
-                self.entry.remove_tag(self._tag)
 
 
 class SourceManager(BaseManager):
@@ -113,6 +116,7 @@ class SourceManager(BaseManager):
         self.values.append(['', '', self._label])
         self.values.append(['all', _("All"), ""])
         self.values.append(['grl-tracker-source', _("Local"), ''])
+        self.props.default_value = 2
 
         grilo.connect('new-source-added', self._add_new_source)
 
@@ -281,6 +285,8 @@ class DropDown(Gtk.Revealer):
     def initialize_filters(self, searchbar):
         self._source_manager = SourceManager(
             'source', _("Sources"), searchbar._search_entry)
+        self._source_manager.connect(
+            "notify::active", self._on_source_manager_value_changed)
 
         self._source_filter.props.manager = self._source_manager
         self._source_filter.connect(
@@ -302,23 +308,24 @@ class DropDown(Gtk.Revealer):
     def _on_selection_changed(self, klass, manager, id_):
         manager.active = id_
 
-        if manager == self._source_manager:
-            is_tracker = self._is_tracker(id_)
-            self._search_filter.props.sensitive = is_tracker
-            self.search_manager.props.active = (
-                'search_all' if not is_tracker else '')
+    @log
+    def _on_source_manager_value_changed(self, klass, value):
+        is_tracker = self._is_tracker(klass.props.active)
+        self._search_filter.props.sensitive = is_tracker
+        self.search_manager.props.active = (
+            'search_all' if not is_tracker else '')
 
     @log
     def _is_tracker(self, grilo_id):
         return grilo_id == "grl-tracker-source"
 
 
-@Gtk.Template(resource_path="/org/gnome/Music/ui/Searchbar.ui")
-class Searchbar(Gtk.SearchBar):
+@Gtk.Template(resource_path="/org/gnome/Music/ui/SearchBar.ui")
+class SearchBar(Gtk.SearchBar):
     """Widget containing the search entry
     """
 
-    __gtype_name__ = 'Searchbar'
+    __gtype_name__ = 'SearchBar'
 
     _search_entry = Gtk.Template.Child()
     _drop_down_button = Gtk.Template.Child()
@@ -327,17 +334,20 @@ class Searchbar(Gtk.SearchBar):
     stack = GObject.Property(type=Gtk.Stack)
 
     def __repr__(self):
-        return '<Searchbar>'
+        return '<SearchBar>'
 
     @log
     def __init__(self):
-        """Initialize the Searchbar"""
+        """Initialize the SearchBar"""
         super().__init__()
 
         self._timeout = None
 
         self._dropdown = DropDown()
         self._dropdown.initialize_filters(self)
+
+        self.connect(
+            "notify::search-mode-enabled", self._search_mode_changed)
         self.connect('notify::search-state', self._search_state_changed)
 
     @Gtk.Template.Callback()
@@ -348,7 +358,8 @@ class Searchbar(Gtk.SearchBar):
     @Gtk.Template.Callback()
     @log
     def _tag_button_clicked(self, entry, tag_):
-        tag_.manager.active = tag_.manager.values[1][BaseModelColumns.ID]
+        default_value = tag_.manager.values[tag_.manager.props.default_value]
+        tag_.manager.props.active = default_value[BaseModelColumns.ID]
         self._search_entry_changed(None)
 
     @Gtk.Template.Callback()
@@ -382,18 +393,9 @@ class Searchbar(Gtk.SearchBar):
         return False
 
     @log
-    def reveal(self, show, clear=True):
-        """Hides or reveals the searchbar
-
-        :param bool show: Whether to show the searchbar
-        :param bool clear: Whether to clear the search entry
-        """
-        self.props.search_mode_enabled = show
-
-        if show:
+    def _search_mode_changed(self, klass, data):
+        if self.props.search_mode_enabled:
             self._search_entry.realize()
-            if clear:
-                self._search_entry.set_text('')
             self._search_entry.grab_focus()
         else:
             self._drop_down_button.set_active(False)
@@ -402,14 +404,14 @@ class Searchbar(Gtk.SearchBar):
     def _search_state_changed(self, klass, data):
         search_state = self.props.search_state
 
-        if search_state == Search.State.NONE:
-            self.reveal(False)
-        elif search_state == Search.State.NO_RESULT:
+        if search_state == Search.State.NO_RESULT:
             self._set_error_style(True)
             self.props.stack.props.visible_child_name = 'emptyview'
-        else:
+        elif search_state == Search.State.RESULT:
             self._set_error_style(False)
             self.props.stack.props.visible_child_name = 'search'
+        elif search_state == Search.State.NONE:
+            self._search_entry.props.text = ""
 
     @log
     def _set_error_style(self, error):
@@ -422,8 +424,3 @@ class Searchbar(Gtk.SearchBar):
             style_context.add_class('error')
         else:
             style_context.remove_class('error')
-
-    @log
-    def toggle(self):
-        """Toggle the searchbar showing"""
-        self.reveal(not self.get_search_mode())
