@@ -32,7 +32,7 @@ from gi.repository.Dazzle import BoldingLabel  # noqa: F401
 from gnomemusic import log
 from gnomemusic import utils
 from gnomemusic.grilo import grilo
-from gnomemusic.playlists import Playlists, StaticPlaylists
+from gnomemusic.playlists import Playlists, SmartPlaylists
 from gnomemusic.widgets.starimage import StarImage  # noqa: F401
 
 
@@ -111,8 +111,8 @@ class SongWidget(Gtk.EventBox):
 
         self.bind_property(
             'selected', self._select_button, 'active',
-            GObject.BindingFlags.BIDIRECTIONAL |
-            GObject.BindingFlags.SYNC_CREATE)
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
         self.bind_property(
             'show-duration', self._duration_label, 'visible',
             GObject.BindingFlags.SYNC_CREATE)
@@ -143,7 +143,7 @@ class SongWidget(Gtk.EventBox):
 
         # TODO: Rework and stop updating widgets from here directly.
         grilo.set_favorite(self._media, favorite)
-        self._playlists.update_static_playlist(StaticPlaylists.Favorites)
+        self._playlists.update_smart_playlist(SmartPlaylists.Favorites)
 
         return True
 
