@@ -33,9 +33,12 @@ from gi.repository import Gtk, Gdk, Gio, GLib, GObject
 from gettext import gettext as _
 
 from gnomemusic import log
+from gnomemusic.grilo import grilo
 from gnomemusic.mediakeys import MediaKeys
-from gnomemusic.player import Player, RepeatMode
+from gnomemusic.player import RepeatMode
+from gnomemusic.playlists import Playlists
 from gnomemusic.query import Query
+from gnomemusic.search import Search
 from gnomemusic.utils import View
 from gnomemusic.views.albumsview import AlbumsView
 from gnomemusic.views.artistsview import ArtistsView
@@ -47,11 +50,9 @@ from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.notificationspopup import NotificationsPopup
 from gnomemusic.widgets.playertoolbar import PlayerToolbar
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
-from gnomemusic.widgets.searchbar import Searchbar
+from gnomemusic.widgets.searchbar import SearchBar
 from gnomemusic.widgets.selectiontoolbar import SelectionToolbar
 from gnomemusic.windowplacement import WindowPlacement
-from gnomemusic.playlists import Playlists
-from gnomemusic.grilo import grilo
 
 import logging
 logger = logging.getLogger(__name__)
@@ -69,9 +70,13 @@ class Window(Gtk.ApplicationWindow):
 
     @log
     def __init__(self, app):
+        """Initialize the main window.
+
+        :param Gtk.Application app: Application object
+        """
         super().__init__(application=app, title=_("Music"))
 
-        self._settings = Gio.Settings.new('org.gnome.Music')
+        self._settings = app.props.settings
         self.add_action(self._settings.create_action('repeat'))
         select_all = Gio.SimpleAction.new('select_all', None)
         select_all.connect('activate', self._select_all)
@@ -81,11 +86,13 @@ class Window(Gtk.ApplicationWindow):
         self.add_action(select_none)
 
         self.set_size_request(200, 100)
-        WindowPlacement(self, self._settings)
+        WindowPlacement(self)
 
         self.prev_view = None
         self.curr_view = None
 
+        self._player = app.props.player
+
         self.notifications_popup = NotificationsPopup()
         self._setup_view()
 
@@ -118,9 +125,21 @@ class Window(Gtk.ApplicationWindow):
     @log
     def _setup_view(self):
         self._box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+
+        self._search = Search()
         self._headerbar = HeaderBar()
-        self._searchbar = Searchbar()
-        self._player = Player(self)
+        self._searchbar = SearchBar()
+        self._search.bind_property(
+            "search-mode-active", self._headerbar, "search-mode-active",
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
+        self._search.bind_property(
+            "search-mode-active", self._searchbar, "search-mode-enabled",
+            GObject.BindingFlags.SYNC_CREATE)
+        self._search.bind_property(
+            "state", self._searchbar, "search-state",
+            GObject.BindingFlags.SYNC_CREATE)
+
         self._player_toolbar = PlayerToolbar(self._player, self)
         selection_toolbar = SelectionToolbar()
         self.views = [None] * len(View)
@@ -131,10 +150,6 @@ class Window(Gtk.ApplicationWindow):
             visible=True,
             can_focus=False)
 
-        self._headerbar.bind_property(
-            "search-mode-enabled", self._searchbar, "search-mode-enabled",
-            GObject.BindingFlags.BIDIRECTIONAL |
-            GObject.BindingFlags.SYNC_CREATE)
         self._searchbar.props.stack = self._stack
         self._headerbar.connect(
             'back-button-clicked', self._switch_back_from_childview)
@@ -146,8 +161,8 @@ class Window(Gtk.ApplicationWindow):
             'selected-items-count', selection_toolbar, 'selected-items-count')
         self.bind_property(
             'selection-mode', self._headerbar, 'selection-mode',
-            GObject.BindingFlags.BIDIRECTIONAL |
-            GObject.BindingFlags.SYNC_CREATE)
+            GObject.BindingFlags.BIDIRECTIONAL
+            | GObject.BindingFlags.SYNC_CREATE)
         self.bind_property(
             'selection-mode', selection_toolbar, 'visible',
             GObject.BindingFlags.SYNC_CREATE)
@@ -174,10 +189,9 @@ class Window(Gtk.ApplicationWindow):
         self._box.pack_start(selection_toolbar, False, False, 0)
         self.add(self._box)
 
-        self._headerbar._search_button.connect(
-            'toggled', self._on_search_toggled)
         selection_toolbar.connect(
             'add-to-playlist', self._on_add_to_playlist)
+        self._search.connect("notify::state", self._on_search_state_changed)
 
         self._headerbar.props.state = HeaderBar.State.MAIN
         self._headerbar.show()
@@ -269,7 +283,14 @@ class Window(Gtk.ApplicationWindow):
         self._stack.set_visible_child(self.views[View.ALBUM])
 
         self.views[View.SEARCH].bind_property(
-            'search-state', self._searchbar, 'search-state')
+            "search-state", self._search, "state",
+            GObject.BindingFlags.SYNC_CREATE)
+        self._search.bind_property(
+            "search-mode-active", self.views[View.SEARCH],
+            "search-mode-active", GObject.BindingFlags.BIDIRECTIONAL)
+        self._search.bind_property(
+            "search-mode-active", self.views[View.ALBUM],
+            "search-mode-active", GObject.BindingFlags.SYNC_CREATE)
 
     @log
     def _select_all(self, action=None, param=None):
@@ -311,7 +332,8 @@ class Window(Gtk.ApplicationWindow):
             if (keyval == Gdk.KEY_f
                     and not self.views[View.PLAYLIST].rename_active
                     and self._headerbar.props.state != HeaderBar.State.SEARCH):
-                self._searchbar.toggle()
+                search_mode = self._search.props.search_mode_active
+                self._search.props.search_mode_active = not search_mode
             # Play / Pause on Ctrl + SPACE
             if keyval == Gdk.KEY_space:
                 self._player.play_pause()
@@ -382,8 +404,8 @@ class Window(Gtk.ApplicationWindow):
             if keyval == Gdk.KEY_Escape:
                 if self.props.selection_mode:
                     self.props.selection_mode = False
-                else:
-                    self._searchbar.reveal(False)
+                elif self._search.props.search_mode_active:
+                    self._search.props.search_mode_active = False
 
         # Open the search bar when typing printable chars.
         key_unic = Gdk.keyval_to_unicode(keyval)
@@ -394,11 +416,11 @@ class Window(Gtk.ApplicationWindow):
                      or modifiers == 0)
                 and not self.views[View.PLAYLIST].rename_active
                 and self._headerbar.props.state != HeaderBar.State.SEARCH):
-            self._searchbar.reveal(True)
+            self._search.props.search_mode_active = True
 
     @log
     def _on_back_button_pressed(self, gesture, n_press, x, y):
-        self.headerbar._on_back_button_clicked()
+        self._headerbar.emit('back-button-clicked')
 
     @log
     def _notify_mode_disconnect(self, data=None):
@@ -411,9 +433,11 @@ class Window(Gtk.ApplicationWindow):
         self.prev_view = self.curr_view
         self.curr_view = stack.get_visible_child()
 
-        if (self.curr_view != self.views[View.SEARCH]
-                and self.curr_view != self.views[View.EMPTY]):
-            self._searchbar.reveal(False)
+        # Disable search mode when switching view
+        search_views = [self.views[View.EMPTY], self.views[View.SEARCH]]
+        if (self.curr_view not in search_views
+                and self._search.props.search_mode_active is True):
+            self._search.props.search_mode_active = False
 
         # Disable the selection button for the EmptySearch and Playlist
         # view
@@ -442,24 +466,13 @@ class Window(Gtk.ApplicationWindow):
             self._stack.set_visible_child(self.views[view_enum])
 
     @log
-    def _on_search_toggled(self, button, data=None):
-        self._searchbar.reveal(
-            button.get_active(), self.curr_view != self.views[View.SEARCH])
-        if (not button.get_active()
-                and (self.curr_view == self.views[View.SEARCH]
-                    or self.curr_view == self.views[View.EMPTY])):
-            child = self.curr_view.get_visible_child()
-            if self._headerbar.props.state == HeaderBar.State.MAIN:
-                # We should get back to the view before the search
-                self._stack.set_visible_child(
-                    self.views[View.SEARCH].previous_view)
-            elif (self.views[View.SEARCH].previous_view == self.views[View.ALBUM]
-                    and child != self.curr_view._album_widget
-                    and child != self.curr_view._artist_albums_widget):
-                self._stack.set_visible_child(self.views[View.ALBUM])
+    def _on_search_state_changed(self, klass, param):
+        if (self._search.props.state != Search.State.NONE
+                or not self.views[View.SEARCH].previous_view):
+            return
 
-            if self.props.selection_mode:
-                self.props.selection_mode = False
+        # Get back to the view before the search
+        self._stack.set_visible_child(self.views[View.SEARCH].previous_view)
 
     @log
     def _switch_back_from_childview(self, klass=None):
